/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'

import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { createContext, useContext, useEffect, useMemo, useState } from 'react'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG)
    console.log('[ShikiProvider]', ...args)
}

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
}

interface ShikiProviderProps {
  children: React.ReactNode
  initialLanguages: LanguageRegistration[]
  initialThemes: ThemeRegistrationAny[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

export function ShikiProvider({ children, initialLanguages, initialThemes }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    const init = async () => {
      try {
        log('Starting highlighter initialization')
        log('Languages:', initialLanguages)
        log('Themes:', initialThemes)

        const engine = createJavaScriptRegexEngine({
          forgiving: true,
          cache: new Map(),
        })

        const instance = await createHighlighterCore({
          // Pass the imported modules directly
          langs: initialLanguages,
          themes: initialThemes,
          engine,
        })

        // Test tokenization
        const test = instance.codeToTokens('const x = 1', {
          lang: 'typescript',
          theme: initialThemes[0]?.name || 'default',
          includeExplanation: true,
        })

        log('Test tokenization:', {
          tokens: test.tokens[0]?.map(t => ({
            content: t.content,
            color: t.color,
            scopes: t.explanation?.[0]?.scopes,
          })),
        })

        setHighlighter(instance)
        setIsReady(true)
      }
      catch (err) {
        console.error('Failed to initialize highlighter:', err)
        setError(err instanceof Error ? err : new Error('Failed to initialize highlighter'))
      }
    }

    init()
  }, [])

  const value = useMemo(() => ({
    highlighter,
    isReady,
    error,
  }), [highlighter, isReady, error])

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}
